
/*
    This file is part of the MoreQt library - A try to make Qt more cute
    Copyright (C) 2001, 2002  Yves Bailly, kafka@kafka-fr.net

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __MQ_DBL_RANGE_CONTROL_H__
#define __MQ_DBL_RANGE_CONTROL_H__ 1

#include <qrangecontrol.h>


class mqDblRangeControlPrivate ;

class mqDblRangeControl 
{
    public:
        mqDblRangeControl() ;
        mqDblRangeControl(double minValue, double maxValue, unsigned nb_steps,
                          unsigned line_step, unsigned page_step, double value) ;
        mqDblRangeControl(double minValue, double maxValue, double step_size,
                          unsigned line_step, unsigned page_step, double value) ;
        mqDblRangeControl(const QRangeControl& rc, unsigned nb_steps) ;
        mqDblRangeControl(const QRangeControl& rc, double step_size) ;

        virtual ~mqDblRangeControl() ;

        void setupQRangeControl (QRangeControl& rc) const ;
        void synchFromQRangeControl (const QRangeControl& rc) ;
        void synchQRangeControl (QRangeControl& rc) const ;

        double dblValue() const ;
        void setValue(double val) ;
        double dblMinValue () const ;
        double dblMaxValue () const ;
        void setRange (double minValue, double maxValue) ;
        double dblRange() const ;
        void setMinValue (double minVal) ;
        void setMaxValue (double maxVal) ;
        double bound (double v) const ;

        double stepSize() const ;
        unsigned nbSteps() const ;
        void setStepSize(double step_size) ;
        void setNbSteps(unsigned nb_steps) ;

        double mapIntValue(unsigned value) const ;
        unsigned mapDblValue(double value) const ;
        unsigned stepDblValue() const ;

    protected:
        void UpdateSteps() ;

    private:
        mqDblRangeControlPrivate * d ;
} ; // class mqDblRangeControl

#endif // __MQ_DBL_RANGE_CONTROL_H__
