
/*
    This file is part of the MoreQt library - A try to make Qt more cute
    Copyright (C) 2001, 2002  Yves Bailly, kafka@kafka-fr.net

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "mq_dbl_spinbox.h"

/*!
    \class mqDblSpinBox
    \brief A QSpinBox-like widget, which works on \a double instead of \a ints.

*/


mqDblSpinBox::mqDblSpinBox (QWidget * parent,
                            const char * name)
            : QSpinBox(parent, name),
              mDblRC(0),
              mFormat('f'),
              mPrec(6)
{
    mDblRC = new mqDblRangeControl(*this, 0.1) ;
    setDblFormat('f') ;
    setDblPrecision(6) ;
    setValidator(0) ;
}

mqDblSpinBox::mqDblSpinBox (double minValue,
                            double maxValue,
                            unsigned nb_steps,
                            double value,
                            int step,
                            QWidget * parent,
                            const char * name)
            : QSpinBox(0, 10, 1, parent, name),
              mDblRC(0),
              mFormat('f'),
              mPrec(6)
{
    mDblRC = new mqDblRangeControl(minValue, maxValue, (unsigned)nb_steps,
        step, 10, minValue) ;
    mDblRC->setupQRangeControl(*this) ;
    setDblFormat('f') ;
    setDblPrecision(6) ;
    setValidator(0) ;
    setValue(value) ;
    connect (this, SIGNAL(valueChanged(int)),
             this, SLOT(IntValueChanged(int))) ;
}

mqDblSpinBox::mqDblSpinBox (double minValue,
                            double maxValue,
                            double step_size,
                            double value,
                            int step,
                            QWidget * parent,
                            const char * name)
            : QSpinBox(0, 10, 1, parent, name),
              mDblRC(0),
              mFormat('f'),
              mPrec(6)
{
    mDblRC = new mqDblRangeControl(minValue, maxValue, step_size,
        step, 10, minValue) ;
    mDblRC->setupQRangeControl(*this) ;
    setDblFormat('f') ;
    setDblPrecision(6) ;
    setValidator(0) ;
    setValue(value) ;
    connect (this, SIGNAL(valueChanged(int)),
             this, SLOT(IntValueChanged(int))) ;
}

mqDblSpinBox::mqDblSpinBox (double minValue,
                            double maxValue,
                            unsigned nb_steps,
                            double value,
                            char dbl_format,
                            int dbl_precision,
                            int step,
                            QWidget * parent,
                            const char * name)
            : QSpinBox(0, 10, 1, parent, name),
              mDblRC(0),
              mFormat('f'),
              mPrec(6)
{
    mDblRC = new mqDblRangeControl(minValue, maxValue, (unsigned)nb_steps,
        step, 10, minValue) ;
    mDblRC->setupQRangeControl(*this) ;
    setDblFormat(dbl_format) ;
    setDblPrecision(dbl_precision) ;
    setValidator(0) ;
    setValue(value) ;
    connect (this, SIGNAL(valueChanged(int)),
             this, SLOT(IntValueChanged(int))) ;
}

mqDblSpinBox::mqDblSpinBox (double minValue,
                            double maxValue,
                            double step_size,
                            double value,
                            char dbl_format,
                            int dbl_precision,
                            int step,
                            QWidget * parent,
                            const char * name)
            : QSpinBox(0, 10, 1, parent, name),
              mDblRC(0),
              mFormat('f'),
              mPrec(6)
{
    mDblRC = new mqDblRangeControl(minValue, maxValue, step_size,
        step, 10, minValue) ;
    mDblRC->setupQRangeControl(*this) ;
    setDblFormat(dbl_format) ;
    setDblPrecision(dbl_precision) ;
    setValidator(0) ;
    setValue(value) ;
    connect (this, SIGNAL(valueChanged(int)),
             this, SLOT(IntValueChanged(int))) ;
}

mqDblSpinBox::~mqDblSpinBox()
{
    delete mDblRC ;
    mDblRC = 0 ;
}

char mqDblSpinBox::dblFormat() const
{ return mFormat ; }

/*!
    \see QString::arg() for the meaning of the format char
*/
void mqDblSpinBox::setDblFormat(char f)
{
    if ( f == mFormat )
        return ;
    if ( (f == 'g') || (f == 'G') ||
         (f == 'e') || (f == 'E') ||
         (f == 'f') )
         mFormat = f ;
}

int mqDblSpinBox::dblPrecision() const
{ return mPrec ; }

/*!
    \see QString::arg() for how precision is used
*/
void mqDblSpinBox::setDblPrecision(int p)
{ mPrec = p ; }

double mqDblSpinBox::dblValue() const
{ return mDblRC->dblValue() ; }

double mqDblSpinBox::dblMinValue () const
{ return mDblRC->dblMinValue() ; }

double mqDblSpinBox::dblMaxValue () const
{ return mDblRC->dblMaxValue() ; }

double mqDblSpinBox::dblRange() const
{ return mDblRC->dblRange() ; }

double mqDblSpinBox::bound (double v) const
{ return mDblRC->bound(v) ; }

double mqDblSpinBox::stepSize() const
{ return mDblRC->stepSize() ; }

unsigned mqDblSpinBox::nbSteps() const
{ return mDblRC->nbSteps() ; }

// signal
//        void dblValueChanged(double val) ;

// virtual public slots
void mqDblSpinBox::setValue(double val)
{
    mDblRC->setValue(val) ;
    mDblRC->synchQRangeControl(*this) ;
}

// virtual public slots
void mqDblSpinBox::setRange (double minValue, double maxValue)
{
    mDblRC->setRange(minValue, maxValue) ;
    mDblRC->setupQRangeControl(*this) ;
}

// virtual public slots
void mqDblSpinBox::setMinValue (double minVal)
{
    mDblRC->setMinValue(minVal) ;
    mDblRC->setupQRangeControl(*this) ;
}

// virtual public slots
void mqDblSpinBox::setMaxValue (double maxVal)
{
    mDblRC->setMaxValue(maxVal) ;
    mDblRC->setupQRangeControl(*this) ;
}

// virtual public slots
void mqDblSpinBox::setStepSize(double step_size)
{
    mDblRC->setStepSize(step_size) ;
    mDblRC->setupQRangeControl(*this) ;
}

void mqDblSpinBox::setNbSteps(unsigned nb_steps)
{
    mDblRC->setNbSteps(nb_steps) ;
    mDblRC->setupQRangeControl(*this) ;
}

// protected slot
void mqDblSpinBox::IntValueChanged(int)
{
    mDblRC->synchFromQRangeControl(*this) ;
    emit valueChanged(mDblRC->dblValue()) ;
}

// virtual protected
QString mqDblSpinBox::mapValueToText (int v)
{
    return QString::number(mDblRC->mapIntValue(v), mFormat, mPrec) ;
}

// virtual protected
int mqDblSpinBox::mapTextToValue (bool * ok)
{
    double val = cleanText().toDouble(ok) ;
    if ( *ok )
        return mDblRC->mapDblValue(val) ;
    else
        return 0 ;
}

#include "mq_dbl_spinbox.moc"