
/*
    This file is part of the MoreQt library - A try to make Qt more cute
    Copyright (C) 2001, 2002  Yves Bailly, kafka@kafka-fr.net

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __MQ_DBL_SPINBOX_H__
#define __MQ_DBL_SPINBOX_H__

#include <qspinbox.h>

#include "mq_dbl_range_control.h"

class mqDblSpinBox : public QSpinBox
{
    Q_OBJECT
    public:
        mqDblSpinBox (QWidget * parent = 0,
                      const char * name = 0) ;
        mqDblSpinBox (double minValue,
                      double maxValue,
                      unsigned nb_steps,
                      double value,
                      int step = 1,
                      QWidget * parent = 0,
                      const char * name = 0) ;
        mqDblSpinBox (double minValue,
                      double maxValue,
                      double step_size,
                      double value,
                      int step = 1,
                      QWidget * parent = 0,
                      const char * name = 0) ;
        mqDblSpinBox (double minValue,
                      double maxValue,
                      unsigned nb_steps,
                      double value,
                      char dbl_format = 'g',
                      int dbl_precision = 6,
                      int step = 1,
                      QWidget * parent = 0,
                      const char * name = 0) ;
        mqDblSpinBox (double minValue,
                      double maxValue,
                      double step_size,
                      double value,
                      char dbl_format = 'g',
                      int dbl_precision = 6,
                      int step = 1,
                      QWidget * parent = 0,
                      const char * name = 0) ;
        ~mqDblSpinBox() ;

        char dblFormat() const ;
        void setDblFormat(char f = 'g') ;
        int dblPrecision() const ;
        void setDblPrecision(int p = 6) ;

        double dblValue() const ;
        double dblMinValue () const ;
        double dblMaxValue () const ;
        double dblRange() const ;
        double bound (double v) const ;

        double stepSize() const ;
        unsigned nbSteps() const ;

    signals:
        void valueChanged(double val) ;

    public slots:
        virtual void setValue(double val) ;
        virtual void setRange (double minValue, double maxValue) ;
        virtual void setMinValue (double minVal) ;
        virtual void setMaxValue (double maxVal) ;
        virtual void setStepSize(double step_size) ;
        virtual void setNbSteps(unsigned nb_steps) ;

    protected:
        virtual QString mapValueToText (int v) ;
        virtual int mapTextToValue (bool * ok) ;

    protected slots:
        void IntValueChanged(int) ;

    private:
        mqDblRangeControl * mDblRC ;
        char mFormat ;
        int mPrec ;

} ; // class mqDblSpinBox

#endif // __MQ_DBL_SPINBOX_H__
